from __future__ import absolute_import
from __future__ import with_statement

import os
from tempfile import mkstemp

from nose.tools import assert_raises

from dbmigrate.migration import MigrationEngine
from pymock import dummy, mock, override, replay, use_pymock, verify

__docformat__ = "restructuredtext"

BASE_ARGS = ['-v', 'test.ini']

def _test_setup_option_parser():
    migrate = MigrationEngine(['-r1'] + BASE_ARGS)
    assert migrate.options.revision == 1
    assert migrate.options.verbose

def prepare_engine():
    eng = MigrationEngine(dummy(), dummy(), dummy())
    eng.connection = mock()
    return eng

@use_pymock
def test_good_up_migration():
    eng = prepare_engine()
    revision = dummy()
    name = "001_good"
    atoms = dummy()
    override(eng, 'apply_up_atoms').expects(atoms, revision, name, [])
    replay()
    eng.apply_migration_up(atoms, revision, name)
    verify()
        
@use_pymock
def test_bad_up_migration():
    eng = prepare_engine()
    revision = dummy()
    name = "001_bogus"
    atoms = dummy()
    override(eng, 'apply_up_atoms').expects(atoms, revision, name, []).raises(Exception)
    override(eng, 'undo_applied_atoms').expects([])
    replay()
    assert_raises(Exception, eng.apply_migration_up, atoms, revision, name)
    verify()

def test_migration_engine_run():
    def install_database():
        (fd, filename) = mkstemp()
        os.close(fd)
        params = {'scheme': 'sqlite', 'filename':  filename}
        packages = ['test.db.migrate']
        params['revision'] = 1
        eng = MigrationEngine()
        eng.run(params, packages)
        if not os.path.exists(filename):
            assert False
        assert len(eng.datasouce.get_applied_revisions()) == 1
        del params['revision'] 
        eng = MigrationEngine()
        eng.run(params, packages)
        assert len(eng.datasouce.get_applied_revisions()) == 2
        params['revision'] = 0
        eng = MigrationEngine()
        eng.run(params, packages)
        assert len(eng.datasource.get_applied_revisions()) == 0
        os.unlink(filename)
